<?php
/**
 * Tribalogic Web Directory & Booking Platform
 *
 * This file is discributed without any warranty whatsoever.
 * It should not be redistributed without the express written
 * consent of Tribalogic Ltd.
 *
 * http://www.tribalogic.net/
 *
 * @copyright Copyright (c) 2000-2012 Tribalogic Ltd
 */

/**
 * Simple stub class to receive the results of the "report"
 */
final class tlReportResults
{
  public $results;
}


/**
 * Static class (just used to keep namespace tidy)
 */
final class tlSearch
{
  /* There should be no need to change these values */
  const cReportName = 'Search_%sIds';
  const cProductType = 'accommodation';
  const cSoapWsdl = 'https://secure.tribalogic.net/webservices/booking/r1/index.wsdl';

  /**
   * The main function to find available entries
   *
   * @param strArrive The arrival date in dd/mm/yyyy format string.
   * @param strDepart The depature date in dd/mm/yyyy format string (optional)
   * @param intPartySize The party size (optional)
   * @return boolean|array Will return false on error or an array of key=>value pairs
                           consisting of the entry id => ('name'=> entry name, 'price=> holiday price)
   */
  public static function findEntries($intAff, $strArrive, $strDepart = false, $intPartySize = null)
  {
    return self::_find('entry', $intAff, $strArrive, $strDepart, $intPartySize);
  }

  /**
   * The main function to find available providers
   *
   * @deprecated Please use tlSearch::findEntries() instead
   * @param strArrive The arrival date in dd/mm/yyyy format string.
   * @param strDepart The depature date in dd/mm/yyyy format string (optional)
   * @param intPartySize The party size (optional)
   * @return boolean|array Will return false on error or an array of key=>value pairs
                           consisting of the provider id => ('name'=> provider name, 'price=> provider price)
   */
  public static function FindProviders($intAff, $strArrive, $strDepart = false, $intPartySize = null)
  {
    return self::_find('provider', $intAff, $strArrive, $strDepart, $intPartySize);
  }

  private static function _find($mode, $intAff, $strArrive, $strDepart = false, $intPartySize = null)
  {
    if (empty($intAff))
      return false;

    $dom_req = new DomDocument('1.0', 'utf-8');

    /* The SOAP request is made up of two parts:
       - Auth
       - Report Request
     */

    // Build up our "auth" XML
    $el_auth = $dom_req->createElement('auth');
    $el_aff = $dom_req->createElement('affiliate_id');
    $el_auth->appendChild($el_aff);
    $el_aff->appendChild($dom_req->createTextNode($intAff));

    $auth = $dom_req->saveXML($el_auth);

    // Build up our request XML
    $el_reports = $dom_req->createElement('reports');
    $el_report = $dom_req->createElement('report');
    $el_reports->appendChild($el_report);

    $el_report->appendChild($dom_req->createElement('name', sprintf(self::cReportName, ucwords($mode))));
    $el_params = $dom_req->createElement('params');
    $el_report->appendChild($el_params);

    $el = $dom_req->createElement(('entry' == $mode ? 'syndicateId' : 'affiliateId'));
    $el->appendChild($dom_req->createTextNode($intAff));
    $el_params->appendChild($el);

    $el = $dom_req->createElement('productType');
    $el->appendChild($dom_req->createTextNode(self::cProductType));
    $el_params->appendChild($el);

    $start_date = self::_formatDateISO8601($strArrive);
    if (false === $start_date)
      return false;

    $el = $dom_req->createElement(('entry' == $mode ? 'startDate' : 'startDate1'));
    $el->appendChild($dom_req->createTextNode($start_date));
    $el_params->appendChild($el);

    // Here we can add a fuzziness to the start date but for now we'll just
    // keep the two dates the same.
    $el = $dom_req->createElement('startDate2');
    $el->appendChild($dom_req->createTextNode($start_date));
    $el_params->appendChild($el);

    $end_date = self::_formatDateISO8601($strDepart);
    if (!empty($end_date))
    {
      $el = $dom_req->createElement('endDate');
      $el->appendChild($dom_req->createTextNode($end_date));
      $el_params->appendChild($el);
    }
    if (is_numeric($intPartySize))
    {
      $el = $dom_req->createElement('partySize');
      $el->appendChild($dom_req->createTextNode((int)$intPartySize));
      $el_params->appendChild($el);
    }

    $query = $dom_req->saveXML($el_reports);

    /* If you want to look at the raw XML of the request you can uncomment
       the following lines: */
    /*
    $dom_req->formatOutput = true;
    echo "==== Auth XML ====\n";
    echo $dom_req->saveXML($el_auth);
    echo "\n\n";
    echo "==== Report Request XML ====\n";
    echo $dom_req->saveXML($el_reports);
    echo "\n\n";
    exit;
    */

    try
    {
      $soap = new SoapClient(self::cSoapWsdl,
        array('classmap' => array('getReport' => 'tlReportResults')));
      $results = $soap->getReport($auth, $query);

      $dom_res = DomDocument::loadXML($results);

      /* If you want to look at the raw XML results you can uncomment the
         following lines: */
      /*
      $dom_res->formatOutput = true;
      echo "==== Report Request XML ====\n";
      echo $dom_res->saveXML();
      echo "\n\n";
      exit;
      */

      $xpath = new DomXPath($dom_res);
      $results = $xpath->query('//'.$mode.('entry' == $mode ? '' : '[@test = 0]'));
      $rv = array();
      foreach ($results as $result)
      {
        $rv[$result->getAttribute('id')] = array(
          'name'  => $result->getAttribute('name'),
          'price' => (float)$result->getAttribute('price'),
        );
      }
      return $rv;
    }
    catch (Exception $e)
    {
      // Do what you want with exception handling....
      return false;
    }

    // Can't get to this point, but if the impossible happens....
    return false;
  }

  /**
   * Small internal utility function to turn a dd/mm/yyyy date to ISO
   *
   * @param strDate A dd/mm/yyyy string date.
   * @return boolean|string An ISO8601 format date or false on error.
   */
  private static function _formatDateISO8601($strDate)
  {
    if (empty($strDate))
      return false;

    // Dates are in the standard ISO9601 format used in XML.
    // yyyy-mm-dd hh:nn:ss+00:00

    list($day, $month, $year) = explode('/', $strDate, 3);

    // Validate the date range a bit!
    if (empty($day) || empty($month) || empty($year))
      return false;

    return sprintf('%04d-%02d-%02d 00:00:00+00:00', $year, $month, $day);
  }
}
